﻿<#
Library Script: Gigaframe.Com.PowerShell.CGI.MDB.ps1
Software: Gigaframe.Com.PowerShell.CGI
Copyright (C) 2022 Gigaframe.com, Robert Demelo
All rights reserved.
Read License terms, conditions and disclaimer
LICENSE.txt
WARRANTY_LIABILITY_DISCLAIMER.txt
THIRD_PARTY_LICENSE_README.txt

WARRANTY AND LIABILITY DISCLAIMER

THIS SOFTWARE AND ANY SUPPLIED AND GENERATED CODE IS 
PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR, 
CONTRIBUTORS OR ANY ASSOCIATED PARTIES BE LIABLE FOR ANY 
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE. 

#>

<#
Connection Strings

MariaDB 10.5+:
"server=127.0.0.1;uid=root;pwd=12345;database=test" 
#>

#Must install "mysql-connector-net-8.0.30.msi" minimally
#Add-Type -AssemblyName D:\dev\psroot\Lib\MySql.Data.dll;
#[System.Reflection.Assembly]::LoadFrom(“D:\dev\psroot\Lib\MySql.Data.dll”);
#[System.Reflection.Assembly]::LoadFrom(“$PSScriptRoot\Lib\MySql.Data.dll”);
#[void][System.Reflection.Assembly]::LoadWithPartialName("MySql.Data");

Using Assembly MySql.Data;
Using Namespace System.Collections.Generic;

[bool] $_MDB_Error = $False;
[string] $_MDB_ErrorMessage = "";


Function MDBQuery
{

    [OutputType([System.Data.DataTable])]
    param(
        [parameter(Mandatory=$true, Position=0)]
        [string] $ConnString,
        [parameter(Mandatory=$true, Position=1)]
        [string] $SQLStatement,
        [parameter(Mandatory=$false, Position=2)]
        [List[KeyValuePair[[string],[object]]]] $Params
    )

    [System.Data.DataSet] $RetDS = [System.Data.DataSet]::new();

    [MySql.Data.MySqlClient.MySqlConnection] $MDB_Con;
    [MySql.Data.MySqlClient.MySqlCommand] $MDB_Cmd;
    [MySql.Data.MySqlClient.MySqlDataAdapter] $MDB_Dpr;

    $_MDB_Error = $False;
    $_MDB_ErrorMessage = "";
        
    Try 
    {

        $MDB_Con = [MySql.Data.MySqlClient.MySqlConnection]::new($ConnString);

        #Load Data
        $MDB_Con.Open();
        $MDB_Cmd = $MDB_Con.CreateCommand();
        $MDB_Cmd.CommandText = $SQLStatement;

        If($Params -ne $Null)
        {
            ForEach($kvp in $Params)
            {
                #Write-Host("Key = $($kvp.Key), Value = $($kvp.Value)");
                $MDB_Cmd.Parameters.Add([MySql.Data.MySqlClient.MySqlParameter]::new($kvp.Key, $kvp.Value));
            }
        }

        $MDB_Dpr = [MySql.Data.MySqlClient.MySqlDataAdapter]::new($MDB_Cmd);
        $RetDS.Reset();
        $MDB_Dpr.Fill($RetDS);
    }
    Catch [System.IO.IOException]
    {
        $_MDB_Error = $True;
        $_MDB_ErrorMessage = $error.Exception;
    }
    Finally
    {
        If($MDB_Cmd -ne $Null)
        {
            $MDB_Cmd.Dispose();
        }

        If($RetDS -ne $Null)
        {
            $RetDS.Dispose();
        }

        If($MDB_Con.State -eq [System.Data.ConnectionState]::Open) 
        {
            $MDB_Con.Close();
        }
    }
    
    #Return $RetDS;
    Write-Output -NoEnumerate $RetDS;
}

Function MDBExecute
{

    [OutputType([bool])]
    param(
        [parameter(Mandatory=$true, Position=0)]
        [string] $ConnString,
        [parameter(Mandatory=$true, Position=1)]
        [string] $SQLStatement,
        [parameter(Mandatory=$false, Position=2)]
        [List[KeyValuePair[[string],[object]]]] $Params
    )

    [bool] $RetVal = $False;
    
    [MySql.Data.MySqlClient.MySqlConnection] $MDB_Con;
    [MySql.Data.MySqlClient.MySqlCommand] $MDB_Cmd;
    [MySql.Data.MySqlClient.MySqlDataAdapter] $MDB_Dpr;

    $_MDB_Error = $False;
    $_MDB_ErrorMessage = "";
        
    Try 
    {

        $MDB_Con = [MySql.Data.MySqlClient.MySqlConnection]::new($ConnString);

        #Load Data
        $MDB_Con.Open();
        $MDB_Cmd = $MDB_Con.CreateCommand();
        $MDB_Cmd.CommandText = $SQLStatement;

        If($Params -ne $Null)
        {
            ForEach($kvp in $Params)
            {
                #Write-Host("Key = $($kvp.Key), Value = $($kvp.Value)");
                $MDB_Cmd.Parameters.Add([MySql.Data.MySqlClient.MySqlParameter]::new($kvp.Key, $kvp.Value));
            }
        }

        $RetRows = $MDB_Cmd.ExecuteNonQuery();

        If($RetRows -gt 0)
        {
            $RetVal = $True;
        }
        
    }
    Catch [System.IO.IOException]
    {
        $RetVal = $False;
        $_MDB_Error = $True;
        $_MDB_ErrorMessage = $error.Exception;
    }
    Finally
    {
        If($MDB_Con.State -eq [System.Data.ConnectionState]::Open) 
        {
            $MDB_Con.Close();
        }
    }
    
    #Return $RetVal;
    Write-Output -NoEnumerate $RetVal;
}

<##>


#Example

<#
$array = @(
@("k1","v1"),
@("k2","v2"),
@("k3","v3")
);
$array.GetType()

ForEach($a in $array)
{
    Write("$($a[0])=$($a[1])");
}
#>

<#

Clear-Host;


[string] $ConnStr = "server=127.0.0.1;uid=dev;pwd=develop123!;database=org";
[string] $SQLTxt = "SELECT `UserID`, `FirstName`, `LastName`, `Email`, `Phone` FROM `USERS` WHERE `FirstName` LIKE @FirstName";

[List[KeyValuePair[[string],[object]]]] $ParamList = 
[List[KeyValuePair[[string],[object]]]]::new();
$ParamList.Add([KeyValuePair[[string],[object]]]::new("@FirstName","R%"));

#[System.Data.DataSet] $Ds = MDBQuery -ConnString $ConnStr -SQLStatement $SQLTxt -Params $ParamList;

$Ds = MDBQuery -ConnString $ConnStr -SQLStatement $SQLTxt -Params $ParamList;

Write-Host("Error: $_MDB_Error");
Write-Host("ErrorMessage: $_MDB_ErrorMessage");

Write-Host("UserID|FirstName|LastName|Email|Phone");
ForEach($r in $Ds.Tables)
{
    Write-Host("$($r.UserID)|$($r.FirstName)|$($r.LastName)|$($r.Email)|$($r.Phone)");
}

#>

<#

ForEach($Dt in $Ds.Tables)
{
    $Columns = "";

    ForEach($DCol in $Dt.Table.Columns)
    {
        $Columns = $Columns + $DCol.ColumnName + "|";
    }
    
    Write-Host($Columns);

    ForEach($DRow in $Dt.Table.Rows)
    {
        $Row = "";

        ForEach($DCol in $Dt.Table.Columns)
        {
            $Row = $Row + $DRow[$DCol.ColumnName] + "|";
        }

        Write-Host($Row);
    } 
}

#>

