﻿<#
Library Script: Gigaframe.Com.PowerShell.CGI.SQL.ps1
Software: Gigaframe.Com.PowerShell.CGI
Copyright (C) 2022 Gigaframe.com, Robert Demelo
All rights reserved.
Read License terms, conditions and disclaimer
LICENSE.txt
WARRANTY_LIABILITY_DISCLAIMER.txt
THIRD_PARTY_LICENSE_README.txt

WARRANTY AND LIABILITY DISCLAIMER

THIS SOFTWARE AND ANY SUPPLIED AND GENERATED CODE IS 
PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR, 
CONTRIBUTORS OR ANY ASSOCIATED PARTIES BE LIABLE FOR ANY 
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE. 

#>

<#
Connection Strings

SQL 2000:
Data Source=myServerAddress;Initial Catalog=myDataBase;User Id=myUsername;Password=myPassword;
 
SQL 2005+:
Data Source=myServerAddress;Initial Catalog=myDataBase;User Id=myUsername;Password=myPassword;
Server=myServerAddress;Database=myDataBase;Trusted_Connection=True;
Server=myServerName\theInstanceName;Database=myDataBase;Trusted_Connection=True;
#>

Using Namespace System.Collections.Generic;

[bool] $_MSSQL_Error = $False;
[string] $_MSSQL_ErrorMessage = "";


Function MSSQLQuery
{

    [OutputType([System.Data.DataTable])]
    param(
        [parameter(Mandatory=$true, Position=0)]
        [string] $ConnString,
        [parameter(Mandatory=$true, Position=1)]
        [string] $SQLStatement,
        [parameter(Mandatory=$false, Position=2)]
        [List[KeyValuePair[[string],[object]]]] $Params
    )

    [System.Data.DataSet] $RetDS = [System.Data.DataSet]::new();

    [System.Data.SqlClient.SqlConnection] $MSSQL_Con;
    [System.Data.SqlClient.SqlCommand] $MSSQL_Cmd;
    [System.Data.SqlClient.SqlDataAdapter] $MSSQL_Dpr;

    $_MSSQL_Error = $False;
    $_MSSQL_ErrorMessage = "";
        
    Try 
    {

        $MSSQL_Con = [System.Data.SqlClient.SqlConnection]::new($ConnString);

        #Load Data
        $MSSQL_Con.Open();
        $MSSQL_Cmd = $MSSQL_Con.CreateCommand();
        $MSSQL_Cmd.CommandText = $SQLStatement;

        If($Params -ne $Null)
        {
            ForEach($kvp in $Params)
            {
                #Write-Host("Key = $($kvp.Key), Value = $($kvp.Value)");
                $MSSQL_Cmd.Parameters.Add([System.Data.SqlClient.SqlParameter]::new($kvp.Key, $kvp.Value));
            }
        }

        $MSSQL_Dpr = [System.Data.SqlClient.SqlDataAdapter]::new($MSSQL_Cmd);
        $RetDS.Reset();
        $MSSQL_Dpr.Fill($RetDS);
    }
    Catch [System.IO.IOException]
    {
        $_MSSQL_Error = $True;
        $_MSSQL_ErrorMessage = $error.Exception;
    }
    Finally
    {
        If($MSSQL_Cmd -ne $Null)
        {
            $MSSQL_Cmd.Dispose();
        }

        If($RetDS -ne $Null)
        {
            $RetDS.Dispose();
        }

        If($MSSQL_Con.State -eq [System.Data.ConnectionState]::Open) 
        {
            $MSSQL_Con.Close();
        }
    }
    
    #Return $RetDS;
    Write-Output -NoEnumerate $RetDS;
}


Function MSSQLExecute
{

    [OutputType([bool])]
    param(
        [parameter(Mandatory=$true, Position=0)]
        [string] $ConnString,
        [parameter(Mandatory=$true, Position=1)]
        [string] $SQLStatement,
        [parameter(Mandatory=$false, Position=2)]
        [List[KeyValuePair[[string],[object]]]] $Params
    )

    [bool] $RetVal = $False;
    
    [System.Data.SqlClient.SqlConnection] $MSSQL_Con;
    [System.Data.SqlClient.SqlCommand] $MSSQL_Cmd;
    [System.Data.SqlClient.SqlDataAdapter] $MSSQL_Dpr;

    $_MSSQL_Error = $False;
    $_MSSQL_ErrorMessage = "";
        
    Try 
    {

        $MSSQL_Con = [System.Data.SqlClient.SqlConnection]::new($ConnString);

        #Load Data
        $MSSQL_Con.Open();
        $MSSQL_Cmd = $MSSQL_Con.CreateCommand();
        $MSSQL_Cmd.CommandText = $SQLStatement;

        If($Params -ne $Null)
        {
            ForEach($kvp in $Params)
            {
                #Write-Host("Key = $($kvp.Key), Value = $($kvp.Value)");
                $MSSQL_Cmd.Parameters.Add([System.Data.SqlClient.SqlParameter]::new($kvp.Key, $kvp.Value));
            }
        }

        $RetRows = $MSSQL_Cmd.ExecuteNonQuery();

        If($RetRows -gt 0)
        {
            $RetVal = $True;
        }
        
    }
    Catch [System.IO.IOException]
    {
        $RetVal = $False;
        $_MSSQL_Error = $True;
        $_MSSQL_ErrorMessage = $error.Exception;
    }
    Finally
    {
        If($MSSQL_Con.State -eq [System.Data.ConnectionState]::Open) 
        {
            $MSSQL_Con.Close();
        }
    }
    
    #Return $RetVal;
    Write-Output -NoEnumerate $RetVal;
}

<##>


#Example

<#

Clear-Host;


[string] $ConnStr = "Server=(local);Database=Org;Trusted_Connection=True";
#[string] $ConnStr = "Server=(local);Database=Org;User ID=dev;Password=develop123!";
[string] $SQLTxt = "SELECT UserID, FirstName, LastName, Email, Phone FROM USERS WHERE FirstName LIKE @FirstName;SELECT * FROM Groups";
[List[KeyValuePair[[string],[object]]]] $ParamList = 
[List[KeyValuePair[[string],[object]]]]::new();
$ParamList.Add([KeyValuePair[[string],[object]]]::new("@FirstName","R%"));

#[System.Data.DataSet] $Ds = MSSQLQuery -ConnString $ConnStr -SQLStatement $SQLTxt -Params $ParamList;

$Ds = MSSQLQuery -ConnString $ConnStr -SQLStatement $SQLTxt -Params $ParamList;

Write-Host("Error: $_MSSQL_Error");
Write-Host("ErrorMessage: $_MSSQL_ErrorMessage");

Write-Host("UserID|FirstName|LastName|Email|Phone");
ForEach($r in $Ds.Tables)
{
    Write-Host("$($r.UserID)|$($r.FirstName)|$($r.LastName)|$($r.Email)|$($r.Phone)");
}

#>

<#
ForEach($Dt in $Ds.Tables)
{
    $Columns = "";

    ForEach($DCol in $Dt.Table.Columns)
    {
        $Columns = $Columns + $DCol.ColumnName + "|";
    }
    
    Write-Host($Columns);

    ForEach($DRow in $Dt.Table.Rows)
    {
        $Row = "";

        ForEach($DCol in $Dt.Table.Columns)
        {
            $Row = $Row + $DRow[$DCol.ColumnName] + "|";
        }

        Write-Host($Row);
    } 
}
#>
