<#
Library Script: Gigaframe.Com.PowerShell.CGI.Web.ps1
Software Title: Gigaframe.Com.PowerShell.CGI
Copyright (C) 2022 Gigaframe.com, Robert Demelo
All rights reserved.
Read License terms, conditions and disclaimer
LICENSE.txt
WARRANTY_LIABILITY_DISCLAIMER.txt
THIRD_PARTY_LICENSE_README.txt

WARRANTY AND LIABILITY DISCLAIMER

THIS SOFTWARE AND ANY SUPPLIED AND GENERATED CODE IS 
PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR, 
CONTRIBUTORS OR ANY ASSOCIATED PARTIES BE LIABLE FOR ANY 
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE. 

#>

Add-Type -AssemblyName System.Web;

[System.Collections.Generic.Dictionary[string, string]] $QueryVars = [System.Collections.Generic.Dictionary[string, string]]::new();
[System.Collections.Generic.Dictionary[string, string]] $PostVars = [System.Collections.Generic.Dictionary[string, string]]::new();

Function GetQuery([string] $VarName)
{   
    [string] $RetVal = "";
    If($QueryVars.Count -gt 0)
    {
        $RetVal = $QueryVars[$VarName];
    }

    Return $RetVal;
}

Function GetPost([string] $VarName)
{
    [string] $RetVal = "";
    If($PostVars.Count -gt 0)
    {
        $RetVal = $PostVars[$VarName];
    }

    Return $RetVal;
}

#Write-Host("Loading ""Gigaframe.PowerShell.CGI.Web.ps1"" <br/>");

# Parse Query String into Array
[string] $QueryData = [System.Environment]::GetEnvironmentVariable("QUERY_STRING");
    
if($QueryData.Length -gt 0)
{
    #Write-Host("QUERY_STRING: " + [System.Environment]::GetEnvironmentVariable("QUERY_STRING") + "<br/>");
    #Write-Host("QUERY_STRING2: " + $env:QUERY_STRING + "<br/>");

    [string[]] $QueryDataKeyValues = $QueryData.Split("&".ToCharArray());
    [string[]] $KeyValue = $null;

    ForEach($QVars in $QueryDataKeyValues)
    {
        $KeyValue = $QVars.Split("=".ToCharArray());
        $QueryVars[$KeyValue[0]] = [System.Web.HttpUtility]::UrlDecode($KeyValue[1]);
    }

    <#
    ForEach($KeyName in $QueryVars.Keys)
    {
        Write-Host("$KeyName=$($QueryVars[$KeyName]) <br/>");
    }
    #>
}

<#
//Posting in C++ using CGI
char * contentSize = getenv("CONENT_LENGTH");
THROWIF(!contentSize);
long dataSize = atol(contentSize);
char *inStream = new char [dataSize];
cin >> inStream;
DoSomething(inStream); 
#>

[int] $ContentLength = [Convert]::ToInt32([System.Environment]::GetEnvironmentVariable("CONTENT_LENGTH"));
if($ContentLength -gt 0)
{
    $buffer = [char[]]::new($ContentLength);
    #$reader = [Console]::OpenStandardInput(); #[Console]::In;
    $reader = [Console]::In;
    $PostDataChar = $reader.Read($buffer,0,$ContentLength); 
    $reader.Close();
    $PostData = [String]::new($buffer);
    #$PostData = [Console]::In.ReadToEnd();
    #Write-Host("PostData: $PostData<br/>");

    # Parse Query String into Array
    [string[]] $PostStringKeyValues = $PostData.Split("&".ToCharArray());
    [string[]] $KeyValue = $null;

    ForEach($PVars in $PostStringKeyValues)
    {
        $KeyValue = $PVars.Split("=".ToCharArray());
        $PostVars[$KeyValue[0]] = [System.Web.HttpUtility]::UrlDecode($KeyValue[1]);
    }

    <#
    ForEach($KeyName in $PostVars.Keys)
    {
        Write-Host("$KeyName=$($PostVars[$KeyName]) <br/>");
    }
    #>
}

#Write-Host("Loaded ""Gigaframe.PowerShell.CGI.Web.ps1"" <br/>");
